	SUBROUTINE GSEGEI(XGSEQ,YGSEQ,ZGSEQ,SUNRA,XGEI,YGEI,ZGEI)
!
!
! rotates the input GSEq vector to GEI coordinates
!
!			name	type	description
!
!  input arguments:	XGSEQ	REAL*4\
!			YGSEQ	REAL*4 >GSEq vector components
!			ZGSEQ	REAL*4/
!			SUNRA	REAL*4	right ascension of sun in degrees
!
! output arguments:	XGEI	REAL*4\
!			YGEI	REAL*4 >GEI vector components
!			ZGEI	REAL*4/
!
!
	REAL*4 XGSEQ,YGSEQ,ZGSEQ,SUNRA,XGEI,YGEI,ZGEI
!
	XGEI = COSD(SUNRA)*XGSEQ - SIND(SUNRA)*YGSEQ
	YGEI = SIND(SUNRA)*XGSEQ + COSD(SUNRA)*YGSEQ
	ZGEI = ZGSEQ
!
	RETURN
	END
!
!
!
	SUBROUTINE GSEGEO(XGSEQ,YGSEQ,ZGSEQ,SOLMSEC,GLAT,GLONG)
!
! rotates the input GSEq vector to GEOGRAPHIC N LAT and E LONG
!
!			name	type	description
!
!  input arguments:	XGSEQ	REAL*4\
!			YGSEQ	REAL*4 >GSEq vector components
!			ZGSEQ	REAL*4/
!			SOLMSEC	INT*4	apparent solar time in milliseconds
!
! output arguments:	GLAT	REAL*4	geographic north latitude in degrees
!			GLONG	REAL*4	geographic east longitude in degrees
!
!
	INTEGER*4 SOLMSEC
	REAL*4 XGSEQ,YGSEQ,ZGSEQ,GLAT,GLONG,R,THETA,PHI
!
	CALL RECSPHD(XGSEQ,YGSEQ,ZGSEQ,R,THETA,PHI)
	GLAT = 90. - THETA
	GLONG = DMOD(PHI-(SOLMSEC-43200000)*.4166667D-5,360.D0)
!
	RETURN
	END
!
!
!
	SUBROUTINE GEIGSE(XGEI,YGEI,ZGEI,SUNRA,XGSEQ,YGSEQ,ZGSEQ)
!
!
! rotates the input GEI vector to GSEq coordinates
!
!			name	type	description
!
!  input arguments:	XGEI	REAL*4\
!			YGEI	REAL*4 >GSEq vector components
!			ZGEI	REAL*4/
!			SUNRA	REAL*4	right ascension of sun in degrees
!
! output arguments:	XGSEQ	REAL*4\
!			YGSEQ	REAL*4 >GSEq vector components
!			ZGSEQ	REAL*4/
!
!
	REAL*4 XGEI,YGEI,ZGEI,SUNRA,XGSEQ,YGSEQ,ZGSEQ
!
	XGSEQ = COSD(SUNRA)*XGEI + SIND(SUNRA)*YGEI
	YGSEQ = -SIND(SUNRA)*XGEI + COSD(SUNRA)*YGEI
	ZGSEQ = ZGEI
!
	RETURN
	END
!
!
!
	SUBROUTINE GEOGSE(GLAT,GLONG,SOLMSEC,XGSEQ,YGSEQ,ZGSEQ)
!
!
! converts input GEOGRAPHIC N LAT and E LONG to GSEq unit vector
!
!			name	type	description
!
!  input arguments:	GLAT	REAL*4	geographic north latitude in degrees
!			GLONG	REAL*4	geographic east longitude in degrees
!			SOLMSEC	INT*4	apparent solar time in milliseconds
!
! output arguments:	XGSEQ	REAL*4\
!			YGSEQ	REAL*4 >GSEq vector components
!			ZGSEQ	REAL*4/
!			
!
	INTEGER*4 SOLMSEC
	REAL*4 XGSEQ,YGSEQ,ZGSEQ,GLAT,GLONG,RLONG
!
	RLONG = GLONG + 360.0*SOLMSEC/864.D5 + 180.0
	RLONG = AMOD(RLONG,360.0)
	CALL SPHDREC(1.,90.-GLAT,RLONG,XGSEQ,YGSEQ,ZGSEQ)
!
	RETURN
	END
!
!
!
	REAL*4 FUNCTION RADIUS(GLAT)
!
!  returns the radius of the Earth in kilometers at the input
!  geocentric latitude, based on an oblate spheroid with
!  A = 6378.164 and flattening of 1/298.2472
!
!			name	type	description
!
!  input arguments:	GLAT	REAL*4	geocentric latitude in degrees
!
!  result:  		RADIUS	REAL*4	radius of Earth in kilometers
!
!
	DOUBLE PRECISION A,B
	REAL*4 GLAT,SINANG,COSANG
	DATA A/6378.164D0/,B/6356.779D0/
!
	SINANG = SIND(GLAT)
	COSANG = SQRT( 1.-SINANG*SINANG )
	RADIUS = SNGL( A*B / DSQRT(A*A*SINANG*SINANG+B*B*COSANG*COSANG) )
!
	RETURN
	END
!
!
!
	SUBROUTINE SPHDREC(R,THETA,PHI,X,Y,Z)
!
!  converts spherical coordinates R, THETA, and PHI to rectangular
!  coordinates X, Y, and Z
!
!			name	type	description
!
!  input arguments:	R	REAL*4	vector length
!			THETA	REAL*4	angle away from Z-axis (degrees)
!			PHI	REAL*4	angle in XY plane (degrees)
!
! output arguments:	X	REAL*4\
!			Y	REAL*4 >rectangular coordinates of vector
!			Z	REAL*4/
!
!
	REAL*4 R,THETA,PHI,X,Y,Z
!
 	X = R * SIND(THETA) * COSD(PHI)
	Y = R * SIND(THETA) * SIND(PHI)
	Z = R * COSD(THETA)
!
	RETURN
	END
!
!
!
	SUBROUTINE RECSPHD(X,Y,Z,R,THETA,PHI)
!
!  converts rectangular coordinates X, Y, and Z to spherical
!  coordinates R, THETA, and PHI
!
!			name	type	description
!
!  input arguments:	X	REAL*4\
!			Y	REAL*4 >rectangular coordinates of vector
!			Z	REAL*4/
!
! output arguments:	R	REAL*4	vector length
!			THETA	REAL*4	angle away from Z-axis (degrees)
!			PHI	REAL*4	angle in XY plane (degrees)
!
!
	REAL*4 X,Y,Z,R,THETA,PHI
!
	PHI = 0.
	R = DSQRT(DBLE(X)*DBLE(X)+DBLE(Y)*DBLE(Y)+DBLE(Z)*DBLE(Z))
	IF (R.GT.0.) GO TO 9
	THETA = 0.
	RETURN
!
  9	THETA = ACOSD( Z/R )
	IF (THETA.EQ.0..OR.THETA.EQ.180.) RETURN
!
	PHI = ATAN2D( Y,X )
	IF (PHI.LT.0.) PHI = PHI + 360.
	RETURN
!
	END
!
!
!
	SUBROUTINE SUN(YEAR,DAY,UTMSEC,GST,SLONG,SRASN,SDEC)
!
!  calculates sidereal time and position of the sun with
!  accuracy of .006 degree (good for years 1901 through 2099)
!
!			name	type	description
!
!  input arguments:	YEAR	INT*2	year		   \
!			DAY	INT*2	day of year         >universal time
!			UTMSEC	INT*4	milliseconds of day/
!
! output arguments:	GST	REAL*4	Greenwich mean sidereal time (degrees)
!			SLONG	REAL*4	longitude along ecliptic     (degrees)
!			SRASN	REAL*4	apparent right ascension     (degrees)
!			SDEC	REAL*4	declination		     (degrees)
!
!
	REAL*8		DJ,FDAY
	INTEGER*2	YEAR,DAY,IYR,YEAR4
	INTEGER*4	UTMSEC
	REAL*4		GST,SLONG,SRASN,SDEC,T,VL,G,OBLIQ,SLP,SINED,
     &			COSND,COTOBL
!
	IYR = YEAR4(YEAR)
!
	FDAY = UTMSEC/864.D5
	DJ = 365.D0*(IYR-1900) + (IYR-1901)/4 + DAY + FDAY - 0.5D0
	T = DJ/36525.0
	VL = DMOD(279.696678+0.9856473354*DJ,360.D0)
	GST = DMOD(279.690983+0.9856473354*DJ+360.0*FDAY+180.0,360.D0)
	G = DMOD(358.475845+0.985600267*DJ,360.D0)
	SLONG = VL + (1.91946-0.004789*T)*SIND(G) + 0.020094*SIND(2.0*G)
	OBLIQ = (23.45229-0.0130125*T)
	SLP = (SLONG-0.005686)
	SINED = SIND(OBLIQ)*SIND(SLP)
	COSND = SQRT(1.0-SINED*SINED)
	SDEC = ATAND(SINED/COSND)
	COTOBL = COSD(OBLIQ)/SIND(OBLIQ)
	SRASN = 180.0 - ATAN2D( COTOBL*SINED/COSND,-COSD(SLP)/COSND)
!
	RETURN
	END
!
!
!
	SUBROUTINE CROSS(X1,Y1,Z1,X2,Y2,Z2,X3,Y3,Z3)
!
!  computes the cross product of vector 1 and vector 2 (v1 X v2) and
!  returns it as vector 3
!
!			name		type	description
!
!  input arguments:	X1,Y1,Z1	REAL*4	vector 1
!			X2,Y2,Z2	REAL*4	vector 2
!
! output arguments:	X3,Y3,Z3	REAL*4  vector 1 X vector 2
!
!
	REAL*4	X1,Y1,Z1,X2,Y2,Z2,X3,Y3,Z3
!
	X3 = Y1*Z2 - Z1*Y2
	Y3 = Z1*X2 - X1*Z2
	Z3 = X1*Y2 - Y1*X2
!
	RETURN
	END
!
!
!
	SUBROUTINE DCROSS(DX1,DY1,DZ1,DX2,DY2,DZ2,DX3,DY3,DZ3)
!
!  double precision version of CROSS; computes the cross product of
!  vector 1 and vector 2 (v1 X v2) and returns it as vector 3
!
!			name		type	description
!
!  input arguments:	DX1,DY1,DZ1	REAL*8	vector 1
!			DX2,DY2,DZ2	REAL*8	vector 2
!
! output arguments:	DX3,DY3,DZ3	REAL*8  vector 1 X vector 2
!
!
	REAL*8	DX1,DY1,DZ1,DX2,DY2,DZ2,DX3,DY3,DZ3
!
	DX3 = DY1*DZ2 - DZ1*DY2
	DY3 = DZ1*DX2 - DX1*DZ2
	DZ3 = DX1*DY2 - DY1*DX2
!
	RETURN
	END
!
!
!
	SUBROUTINE NRMLIZ(X1,Y1,Z1,X2,Y2,Z2)
!
!  normalizes vector 1 and returns it as vector 2
!
!			name		type	description
!
!  input arguments:	X1,Y1,Z1	REAL*4	vector to be normalized
!
! output arguments:	X2,Y2,Z2	REAL*4	(X1,Y1,Z1) normalized
!
!
	REAL*4	X1,Y1,Z1,X2,Y2,Z2
	REAL*8	DX1,DY1,DZ1,DR
!
	DX1 = DBLE(X1)
	DY1 = DBLE(Y1)
	DZ1 = DBLE(Z1)
!
	DR = DSQRT( DX1*DX1+DY1*DY1+DZ1*DZ1 )
	X2 = SNGL(DX1/DR)
	Y2 = SNGL(DY1/DR)
	Z2 = SNGL(DZ1/DR)
!
	RETURN
	END
!
!
!
	SUBROUTINE DNRMLIZ(DX1,DY1,DZ1,DX2,DY2,DZ2)
!
!  double precision version of NRMLIZ; normalizes vector 1 and
!  returns it as vector 2
!
!			name		type	description
!
!  input arguments:	DX1,DY1,DZ1	REAL*8	vector to be normalized
!
! output arguments:	DX2,DY2,DZ2	REAL*8	(DX1,DY1,DZ1) normalized
!
!
	REAL*8	DX1,DY1,DZ1,DX2,DY2,DZ2,DR
!
	DR = DSQRT( DX1*DX1+DY1*DY1+DZ1*DZ1 )
	DX2 = DX1/DR
	DY2 = DY1/DR
	DZ2 = DZ1/DR
!
	RETURN
	END
!
!
!
	SUBROUTINE QUAD(QA,QB,QC,X1,X2,N)
!
!  solves the quadratic equation QA*X**2 + QB*X + QC = 0, returning
!  as many solutions as are possible
!
!			name	type	description
!
!  input arguments:	QA	REAL*4	coefficient of X**2
!			QB	REAL*4	coefficient of X**1
!			QC	REAL*4	coefficient of X**0
!
! output arguments:	X1	REAL*4	first of one or two possible solutions
!			X2	REAL*4	second of two possible solutions
!			N	INT*2	number of solutions, 0, 1, or 2
!
!
	REAL*4		 QA,QB,QC,X1,X2
	INTEGER*2	 N
	DOUBLE PRECISION FOURAC,BXB
!
	IF (QA.EQ.0.) GO TO 10
	FOURAC = 4.D0*DBLE(QA)*DBLE(QC)
	BXB = DBLE(QB)*DBLE(QB)
	IF (BXB-FOURAC) 10,20,30
!---no solutions
 10	N = 0
	RETURN
!---one solution
 20	N = 1
	X1 = -QB / (2.*QA)
	RETURN
!---two solutions
 30	N = 2
	X1 = SNGL( (-QB - DSQRT(BXB-FOURAC))/(2.D0*QA) )
	X2 = SNGL( (-QB + DSQRT(BXB-FOURAC))/(2.D0*QA) )
	RETURN
!
	END
!
!
!
	SUBROUTINE DQUAD(QA,QB,QC,X1,X2,N)
!
!  solves in double precision the quadratic equation
!  QA*X**2 + QB*X + QC = 0, returning as many solutions as are possible
!
!			name	type	description
!
!  input arguments:	QA	REAL*8	coefficient of X**2
!			QB	REAL*8	coefficient of X**1
!			QC	REAL*8	coefficient of X**0
!
! output arguments:	X1	REAL*8	first of one or two possible solutions
!			X2	REAL*8	second of two possible solutions
!			N	INT*2	number of solutions, 0, 1, or 2
!
!
	REAL*8		 QA,QB,QC,X1,X2,FOURAC,BXB
	INTEGER*2	 N
!
	IF (QA.EQ.0.D0) GO TO 10
	FOURAC = 4.D0*QA*QC
	BXB = QB*QB
	IF (BXB-FOURAC) 10,20,30
!---no solutions
 10	N = 0
	RETURN
!---one solution
 20	N = 1
	X1 = -QB / (2.D0*QA)
	RETURN
!---two solutions
 30	N = 2
	X1 = (-QB - DSQRT(BXB-FOURAC))/(2.D0*QA)
	X2 = (-QB + DSQRT(BXB-FOURAC))/(2.D0*QA)
	RETURN
!
	END
!
!
!
	SUBROUTINE EQTIME(YEAR,DOY,UTMSEC,SOLMSEC)
!
!  converts input universal time to apparent solar time
!
!			name	type	description
!
!  input arguments:	YEAR	INT*2	year
!			DOY	INT*2	day of year
!			UTMSEC	INT*4	universal time in milliseconds
!  output argument:	SOLMSEC	INT*4	apparent solar time in milliseconds
!
!
	INTEGER*4	UTMSEC,SOLMSEC,DAYS
	REAL*4		ANG,E
	INTEGER*2	YEAR,DOY,YR,YEAR4
!
	YR = YEAR4(YEAR)
	DAYS = DOY + (YR-1981)*365 + (YR-1981)/4 
	ANG = 279.58 + 0.985647*(DAYS+DBLE(UTMSEC)/864.D5)
	E = -104.7*SIND(ANG) + 596.2*SIND(2.*ANG) + 4.3*SIND(3.*ANG)
     &    - 12.7*SIND(4.*ANG) - 429.3*COSD(ANG) - 2.0*COSD(2.*ANG)
     &    + 19.3*COSD(3.*ANG)
	SOLMSEC = UTMSEC + NINT(E*1.D3)
!
	RETURN
	END
!
!
!
	INTEGER*2 FUNCTION YEAR4(YEAR)
!
!  converts year to full 4-digit year number; input year less then 51
!  is assumed to be 2000 + year; input year from 51 to 99 is assumed
!  to be 1900 + year
!
!			name	type	description
!
!   input argument:	YEAR	INT*2	2-digit or 4-digit year number
!	    result:	YEAR4	INT*2	4-digit year number
!
!
	INTEGER*2	YEAR
!
	IF (YEAR.LT.100) THEN
	    IF (YEAR.GT.50) THEN
		YEAR4 = 1900 + YEAR
	    ELSE
		YEAR4 = 2000 + YEAR
	    ENDIF
	ELSE
	    YEAR4 = YEAR
	ENDIF
!
	RETURN
	END
