	SUBROUTINE GSE2VW(XGSEQ,YGSEQ,ZGSEQ,XS,YS,VIS)
!
!  converts GEOCENTRIC SOLAR EQUITORIAL coordinates to view coordinates;
!  sets VIS to .FALSE. if the point is not visible
!
!			name	type	description
!
!   input arguments:	XGSEQ	REAL*4\ 
!			YGSEQ	REAL*4 >GSEq position vector in km.
!			ZGSEQ	REAL*4/
!  output arguments:	XS	REAL*4	horizontal view coordinate
!			YS	REAL*4	vertical view coordinate
!			VIS	LOG*1	.TRUE. if input point is visible
!					  to spacecraft; .FALSE. if not
!
!
        COMMON /IMCOORD/IMXMIN,IMXMAX,IMYMAX,STEP,PIXWD,PIXHT,
     &                  SP,ALTF,NSPIN,TSPIN,UTNC1,SOLMSEC1,MDIREC
	COMMON /WINDOW/	WNXMIN,WNXMAX,WNYMIN,WNYMAX
	COMMON /MOTION/	POSITION,SPINANG,XP1,SPINX,SPINY,SPINZ,
     &			SUNX,SUNZ
!
	INTEGER*2	MDIREC,IPOS
	INTEGER*4	UTNC1,NSPIN,TSPIN,SOLMSEC1
	REAL*4		IMXMIN,IMXMAX,IMYMAX,STEP,
     &			PIXWD,PIXHT,WNXMIN,WNXMAX,WNYMIN,WNYMAX,
     &			POSITION(1026,3),SPINANG(1026),XP1,SP,ALTF,
     &			SPINX,SPINY,SPINZ,SUNX,SUNZ,PLX,PLY,PLZ,
     &			PSCX,PSCY,PSCZ,PGX,PGY,PGZ,XGSEQ,YGSEQ,ZGSEQ,
     &			XS,YS
	LOGICAL*1	VIS
!
	VIS = .TRUE.
!
!---get best position index
	CALL GSESCPOS(XGSEQ,YGSEQ,ZGSEQ,IPOS)
	IF (IPOS.LT.1.OR.IPOS.GT.NINT(WNXMAX-XP1)+2) THEN
	    VIS = .FALSE.
	    RETURN
	ENDIF
!
!---get vector from s/c to point in view coordinates
	CALL GSESC2VW(XGSEQ-POSITION(IPOS,1),YGSEQ-POSITION(IPOS,2),
     &			ZGSEQ-POSITION(IPOS,3),IPOS,XS,YS)
!
!---see if it is behind the limb
 13	CALL LMPLAN(IPOS,PLX,PLY,PLZ)
	CALL NRMLIZ(POSITION(IPOS,1)-PLX,POSITION(IPOS,2)-PLY,
     &	  POSITION(IPOS,3)-PLZ,PSCX,PSCY,PSCZ)
	CALL NRMLIZ(XGSEQ-PLX,YGSEQ-PLY,ZGSEQ-PLZ,PGX,PGY,PGZ)
	IF (PSCX*PGX+PSCY*PGY+PSCZ*PGZ.LT.0.) THEN
	    VIS = .FALSE.
	    RETURN
	ENDIF
!
!---check for window boundaries
	IF (XS.LT.WNXMIN) VIS = .FALSE.
	IF (XS.GT.WNXMAX) VIS = .FALSE.
	IF (YS.LT.WNYMIN) VIS = .FALSE.
	IF (YS.GT.WNYMAX) VIS = .FALSE.
!
	RETURN
	END
!
!
!
	SUBROUTINE GEI2VW(XGEI,YGEI,ZGEI,XS,YS,VIS)
!
!
!  converts GEOCENTRIC EQUITORIAL INERTIAL coordinates to view
!  coordinates; sets VIS to .FALSE. if the point is not visible
!
!			name	type	description
!
!   input arguments:	XGEI	REAL*4\ 
!			YGEI	REAL*4 >GEI position vector in km.
!			ZGEI	REAL*4/
!  output arguments:	XS	REAL*4	horizontal view coordinate
!			YS	REAL*4	vertical view coordinate
!			VIS	LOG*1	.TRUE. if input point is visible
!					  to spacecraft; .FALSE. if not
!
!
	COMMON /MAFREC/	HEADER,SCAN
!
	BYTE		HEADER(404),SCAN(174)
	INTEGER*4	SUN(3)
	REAL*4		XGEI,YGEI,ZGEI,GX,GY,GZ,XS,YS,TRUEXS
	REAL*8		SCX,SCY,SCZ,ZANG
	LOGICAL*1	VIS
!
	EQUIVALENCE (HEADER(173),SUN(1))
!
!---rotate the vector to GSEq
	CALL GEIGSE(XGEI,YGEI,ZGEI,ATAN2D(SUN(2)/1.E6,SUN(1)/1.E6),
     &		GX,GY,GZ)
!
!---get view coordinates for the GSEq point
	CALL GSE2VW(GX,GY,GZ,XS,YS,VIS)
!
	RETURN
	END
!
!
!
	SUBROUTINE RADC2VW(RA,DC,XS,YS,VIS)
!
!  finds the view coordinates of the point in the star background
!  indicated by the input right ascension and declination;
!  sets VIS to .FALSE. if the point is not visible
!
!			name	type	description
!
!   input arguments:	RA	REAL*4	right ascension in degrees
!			DC	REAL*4	declination in degrees
!  output arguments:	XS	REAL*4	horizontal view coordinate
!			YS	REAL*4	vertical view coordinate
!			VIS	LOG*1	.TRUE. if input point is visible
!					  to spacecraft; .FALSE. if not
!
!
        COMMON /IMCOORD/IMXMIN,IMXMAX,IMYMAX,STEP,PIXWD,PIXHT,
     &                  SP,ALTF,NSPIN,TSPIN,UTNC1,SOLMSEC1,MDIREC
	COMMON /WINDOW/	WNXMIN,WNXMAX,WNYMIN,WNYMAX
	COMMON /MOTION/	POSITION,SPINANG,XP1,SPINX,SPINY,SPINZ,
     &			SUNX,SUNZ
	COMMON /MAFREC/	HEADER,SCAN
!
	BYTE		HEADER(404),SCAN(174)
	INTEGER*2	MDIREC,IPOS
	INTEGER*4	NSPIN,TSPIN,UTNC1,SUN(3),SOLMSEC1
	REAL*4		IMXMIN,IMXMAX,IMYMAX,STEP,SP,ALTF,
     &			PIXWD,PIXHT,WNXMIN,WNXMAX,WNYMIN,WNYMAX,
     &			POSITION(1026,3),SPINANG(1026),XP1,
     &			SPINX,SPINY,SPINZ,SUNX,SUNZ,RA,DC,
     &			XGEI,YGEI,ZGEI,GX,GY,GZ,XS,YS,TRUEXS
	REAL*8		SCX,SCY,SCZ
	LOGICAL*1	VIS
!
	EQUIVALENCE (HEADER(173),SUN(1))
!
!---rotate to a GSEq vector
	CALL SPHDREC(1.,90.-DC,RA,XGEI,YGEI,ZGEI)
	CALL GEIGSE(XGEI,YGEI,ZGEI,ATAN2D(SUN(2)/1.E6,SUN(1)/1.E6),
     &		GX,GY,GZ)
!
!---get s/c position index
	VIS = .TRUE.
	CALL SCPOS(GX,GY,GZ,IPOS)
	IF (IPOS.LT.1.OR.IPOS.GT.NINT(WNXMAX-XP1)+2) THEN
	    VIS = .FALSE.
	    RETURN
	ENDIF
!	
!---get view coordinates for the direction
	CALL GSESC2VW(GX,GY,GZ,IPOS,XS,YS)
!
!---check for window boundaries
	IF (XS.LT.WNXMIN) VIS = .FALSE.
	IF (XS.GT.WNXMAX) VIS = .FALSE.
	IF (YS.LT.WNYMIN) VIS = .FALSE.
	IF (YS.GT.WNYMAX) VIS = .FALSE.
!
	RETURN
	END
!
!
!
	SUBROUTINE PXLOC(XS,YS,PMLC,INDEX)
!
!  returns the mirror location count and the pixel index for the
!  input view coordinates
!
!			name	type	description
!
!   input arguments:	XS	REAL*4	horizontal view coordinate
!			YS	REAL*4	vertical view coordinate
!  output arguments:	PMLC	INT*2	mirror location count of scanline
!			INDEX	INT*2	pixel position in scanline
!
!
        COMMON /IMAGE/  IMFILE,FCTDWN,FCTACR,NPHOT,CENTRMLC,PIXOFF
        COMMON /IMCOORD/IMXMIN,IMXMAX,IMYMAX,STEP,PIXWD,PIXHT,
     &                  SP,ALTF,NSPIN,TSPIN,UTNC1,SOLMSEC1,MDIREC
        COMMON /SCANS/  LNCOR,MLCS,NPIXL,SCANPIX
	COMMON /MAFREC/	HEADER,SCAN
!
	CHARACTER*128	IMFILE
	BYTE		HEADER(404),SCAN(174),SCANPIX(150,121)
	INTEGER*2	NPHOT,CENTRMLC,PIXOFF,PMLC,INDEX,MDIREC,LN,
     &			MLCS(121),NPIXL(121)
	INTEGER*4	NSPIN,TSPIN,FIRSTMLC,LASTMLC,UTNC1,SOLMSEC1
	REAL*4		IMXMIN,IMXMAX,IMYMAX,XS,YS,FCTDWN,FCTACR,STEP,
     &			PIXWD,PIXHT,Y,LNCOR(2,121),SP,ALTF
!
	EQUIVALENCE (HEADER(41),FIRSTMLC),(HEADER(45),LASTMLC)
!
	IF (NPHOT.EQ.3) THEN
	    PMLC = MIN(FIRSTMLC,LASTMLC) + NINT(XS-IMXMIN)
	ELSE
	    PMLC = MAX(FIRSTMLC,LASTMLC) - NINT(XS-IMXMIN)
	ENDIF
	LN = ABS(PMLC-FIRSTMLC) + 1
	Y = -LNCOR(1,LN) + IMYMAX - YS
	IF (NINT(Y).LT.75) Y = Y - LNCOR(2,LN)
	INDEX = NINT(Y) + 1
!
	RETURN
	END
!
!
!
	SUBROUTINE LOCPX(PMLC,INDEX,XS,YS)
!
!  returns the view coordinates for the input mirror location count and
!  pixel index
!
!			name	type	description
!
!   input arguments:	PMLC	INT*2	mirror location count of scanline
!			INDEX	INT*2	pixel position in scanline
!  output arguments:	XS	REAL*4	horizontal view coordinate
!			YS	REAL*4	vertical view coordinate
!
!
        COMMON /IMAGE/  IMFILE,FCTDWN,FCTACR,NPHOT,CENTRMLC,PIXOFF
        COMMON /IMCOORD/IMXMIN,IMXMAX,IMYMAX,STEP,PIXWD,PIXHT,
     &                  SP,ALTF,NSPIN,TSPIN,UTNC1,SOLMSEC1,MDIREC
        COMMON /SCANS/  LNCOR,MLCS,NPIXL,SCANPIX
	COMMON /MAFREC/	HEADER,SCAN
!
	BYTE		HEADER(404),SCAN(174),SCANPIX(150,121)
	CHARACTER*128	IMFILE
	INTEGER*2	NPHOT,CENTRMLC,PIXOFF,PMLC,INDEX,MDIREC,LN,
     &			MLCS(121),NPIXL(121)
	INTEGER*4	NSPIN,TSPIN,FIRSTMLC,LASTMLC,UTNC1,SOLMSEC1
	REAL*4		IMXMIN,IMXMAX,IMYMAX,XS,YS,FCTDWN,FCTACR,STEP,
     &			PIXWD,PIXHT,LNCOR(2,121),SP,ALTF
!
	EQUIVALENCE (HEADER(41),FIRSTMLC),(HEADER(45),LASTMLC)
!
	IF (NPHOT.EQ.3) THEN
	    XS = IMXMIN + PMLC - MIN(FIRSTMLC,LASTMLC)
	ELSE
	    XS = IMXMIN + MAX(FIRSTMLC,LASTMLC) - PMLC
	ENDIF
	LN = ABS(PMLC-FIRSTMLC) + 1
	YS = IMYMAX - LNCOR(1,LN) - INDEX + 1
	IF (INDEX.LT.76) YS = YS - LNCOR(2,LN)
!
	RETURN
	END
!
!
!	
	SUBROUTINE COUNTS(PMLC,INDEX,CCOUNT,UCOUNT,SCLINT)
!
!  returns compressed and uncompressed pixel count and line-of-sight
!  intensity in kilorayleighs for the input mirror location count and
!  pixel index (conversion to kilorayleighs uses preflight values of
!  instrument calibrations; no corrections are provided for inflight
!  radiation degradation; no corrections are made for background and
!  albedo contamination)
!
!			name	type	description
!
!   input arguments:	PMLC	INT*2	mirror location count of scanline
!			INDEX	INT*2	pixel position in scanline
!  output arguments:	CCOUNT	INT*2	compressed count
!			UCOUNT	INT*2	uncompressed count
!			SCLINT	REAL*4	spacecraft line-of-sight intensity
!					  in kilorayleighs
!
        COMMON /IMCOORD/IMXMIN,IMXMAX,IMYMAX,STEP,PIXWD,PIXHT,
     &                  SP,ALTF,NSPIN,TSPIN,UTNC1,SOLMSEC1,MDIREC
        COMMON /SCANS/  LNCOR,MLCS,NPIXL,SCANPIX
!
	BYTE		HEADER(404),SCAN(174),SCANPIX(150,121)
	INTEGER*2	PMLC,INDEX,MLCS(121),NPIXL(121),CCOUNT,UCOUNT,
     &			ICOUNT,MDIREC,LN
	INTEGER*4	NSPIN,TSPIN,UTNC1,SOLMSEC1,FIRSTMLC
	REAL*4		SP,ALTF,XS,YS,SCLINT,IMXMIN,IMXMAX,IMYMAX,
     &			STEP,PIXWD,PIXHT,LNCOR(2,121)
!
	EQUIVALENCE (HEADER(41),FIRSTMLC)
!
!---get the pixel counts and intensity
	LN = ABS(PMLC-FIRSTMLC) + 1
	IF (INDEX.LT.151.AND.SCANPIX(INDEX,LN).NE.-1) THEN
	    CCOUNT = ZEXT(SCANPIX(INDEX,LN))
	    UCOUNT = ICOUNT(CCOUNT)
	    SCLINT = UCOUNT/SP
	ELSE
	    CCOUNT = -1
	ENDIF
!
	END
!
!
!
	REAL*4 FUNCTION SZA(XS,YS,ALT)
!
!  returns the solar zenith angle at the point defined by the
!  input view coordinates and altitude in kilmeters.
!
!
!			name	type	description
!
!   input arguments:	XS	REAL*4	horizontal view coordinate
!			YS	REAL*4	vertical view coordinate
!			ALT	REAL*4	altitude in kilometers
!	     result:	SZA	REAL*4	solar zenith angle in degrees
!					  at input point
!
!
	COMMON /MOTION/	POSITION,SPINANG,XP1,SPINX,SPINY,SPINZ,
     &			SUNX,SUNZ
!
	REAL*4		POSITION(1026,3),SPINANG(1026),XP1,
     &			SPINX,SPINY,SPINZ,SUNX,SUNZ,XS,YS,ALT,
     &			X1,Y1,Z1,X2,Y2,Z2,PX,PY,PZ
	INTEGER*2	NP
!
	SZA = -1.
	CALL POINT(XS,YS,ALT,NP,X1,Y1,Z1,X2,Y2,Z2)
	IF (NP.EQ.0) RETURN
	CALL NRMLIZ(X1,Y1,Z1,PX,PY,PZ)
	SZA = ACOSD( SUNX*PX + SUNZ*PZ )
!
	RETURN
	END
!
!
!
	REAL*4 FUNCTION DEZA(XS,YS,ALT)
!
!  returns the s/c zenith angle at the point defined by the
!  input view coordinates and altitude
!
!			name	type	description
!
!   input arguments:	XS	REAL*4	horizontal view coordinate
!			YS	REAL*4	vertical view coordinate
!			ALT	REAL*4	altitude in kilometers
!	     result:	DEZA	REAL*4	spacecraft zenith angle in degrees
!					  at input point
!
!
	COMMON /MOTION/	POSITION,SPINANG,XP1,SPINX,SPINY,SPINZ,
     &			SUNX,SUNZ
!
	REAL*4		POSITION(1026,3),SPINANG(1026),XP1,
     &			SPINX,SPINY,SPINZ,SUNX,SUNZ,XS,YS,ALT,
     &			X1,Y1,Z1,X2,Y2,Z2,PX,PY,PZ,P2SCX,P2SCY,P2SCZ
	INTEGER*2	NP,IPOS,XPOS
!
	DEZA = -1.
	CALL POINT(XS,YS,ALT,NP,X1,Y1,Z1,X2,Y2,Z2)
	IF (NP.EQ.0) RETURN
	IPOS = XPOS(XS,YS)
	CALL NRMLIZ(POSITION(IPOS,1)-X1,POSITION(IPOS,2)-Y1,
     &	    POSITION(IPOS,3)-Z1,P2SCX,P2SCY,P2SCZ)
	CALL NRMLIZ(X1,Y1,Z1,PX,PY,PZ)
	DEZA = ACOSD( PX*P2SCX + PY*P2SCY + PZ*P2SCZ )
!
	RETURN
	END
!
!
!
	SUBROUTINE VWDRGEI(XS,YS,XGEI,YGEI,ZGEI)
!
!  computes GEI unit vector from s/c for direction of input view coordinates
!
!			name	type	description
!
!   input arguments:	XS	REAL*4	horizontal view coordinate
!			YS	REAL*4	vertical view coordinate
!  output arguments:	XGEI	REAL*4\ 
!			YGEI	REAL*4 >GEI unit vector
!			ZGEI	REAL*4/
!
!
        COMMON /IMCOORD/IMXMIN,IMXMAX,IMYMAX,STEP,PIXWD,PIXHT,
     &                  SP,ALTF,NSPIN,TSPIN,UTNC1,SOLMSEC1,MDIREC
	COMMON /MAFREC/	HEADER,SCAN
!
	BYTE		HEADER(404),SCAN(174)
	INTEGER*2	IPOS,MDIREC
	INTEGER*4	UTNC1,NSPIN,TSPIN,SUN(3),SOLMSEC1
	REAL*4		IMXMIN,IMXMAX,IMYMAX,STEP,PIXWD,PIXHT,SP,ALTF,
     &			XS,YS,XGEI,YGEI,ZGEI,SCX,SCY,SCZ,GX,GY,GZ
!
	EQUIVALENCE (HEADER(173),SUN(1))
!
!---get look angle in GSEq coordinates
	CALL VW2GSESC(XS,YS,GX,GY,GZ)
!
!---convert vector from GSEq to GEI
	CALL GSEGEI(GX,GY,GZ,ATAN2D(SUN(2)/1.E6,SUN(1)/1.E6),
     &		XGEI,YGEI,ZGEI)
!
	RETURN
	END
!
!
!
	SUBROUTINE LSDIST(XGSEQ,YGSEQ,ZGSEQ,RLS,ALTLS)
!
!  for the look direction described by the input GSEq vector,
!  returns the smallest radial distance from Earth and altitude
!  of the line of sight
!
!			name	type	description
!
!   input arguments:	XGSEQ	REAL*4\ 
!			YGSEQ	REAL*4 >GSEq vector of s/c sight direction
!			ZGSEQ	REAL*4/
!  output arguments:	RLS	REAL*4	least radial distance of
!					  line of sight
!			ALTLS	REAL*4	least altitude of line of sight
!
!
	COMMON /WINDOW/	WNXMIN,WNXMAX,WNYMIN,WNYMAX
	COMMON /MOTION/	POSITION,SPINANG,XP1,SPINX,SPINY,SPINZ,
     &			SUNX,SUNZ
!
	INTEGER*2	IPOS
	REAL*4		WNXMIN,WNXMAX,WNYMIN,WNYMAX,
     &			POSITION(1026,3),SPINANG(1026),XP1,
     &			SPINX,SPINY,SPINZ,SUNX,SUNZ,XGSEQ,YGSEQ,ZGSEQ,
     &			RLS,ALTLS,GX,GY,GZ,PX,PY,PZ,ANGLE,G,RP,
     &			RX,RY,RZ,RADIUS
!
!---get s/c position index
	CALL SCPOS(XGSEQ,YGSEQ,ZGSEQ,IPOS)
	IF (IPOS.LT.1.OR.IPOS.GT.NINT(WNXMAX-XP1)+2) THEN
	    RLS = 0.
	    ALTLS = 0.
	    RETURN
	ENDIF
!
!---compute angle between position vector and line-of-sight vector
	CALL NRMLIZ(-POSITION(IPOS,1),-POSITION(IPOS,2),-POSITION(IPOS,3),
     &	  PX,PY,PZ)
	CALL NRMLIZ(XGSEQ,YGSEQ,ZGSEQ,GX,GY,GZ)
	ANGLE = ACOS( PX*GX + PY*GY + PZ*GZ )
!
!---find the closest radial distance
	RP = SQRT( POSITION(IPOS,1)*POSITION(IPOS,1) + POSITION(IPOS,2)*
     &	  POSITION(IPOS,2) + POSITION(IPOS,3)*POSITION(IPOS,3) )
	RLS = RP*SIN(ANGLE)
!
!---change to altitude
	G = RP*COS(ANGLE)
	RX = POSITION(IPOS,1) + G*GX
	RY = POSITION(IPOS,2) + G*GY
	RZ = POSITION(IPOS,3) + G*GZ
	ALTLS = RLS - RADIUS(ASIND(RZ/SQRT(RX*RX+RY*RY+RZ*RZ)))
!
	RETURN
	END
!
!
!
	SUBROUTINE POINT(XS,YS,ALT,NP,X1,Y1,Z1,X2,Y2,Z2)
!
!  for line of sight indicated by the input view coordinates,
!  computes GSEq points of intersection with the Earth spheroid
!  at the altitude ALT; sets NP to the number of points (0, 1, or 2)
!  and sets (X1,Y1,Z1) to the point closer to the s/c
!
!			name	type	description
!
!   input arguments:	XS	REAL*4	horizontal view coordinate
!			YS	REAL*4	vertical view coordinate
!			ALT	REAL*4	altitude in kilometers
!  output arguments:	NP	INT*2	number of points of intersection -
!					  0, 1, or 2; NP is set to 0 if
!					  computation can't be done
!			X1\	REAL*4\	GSEq position of line-of-sight
!			Y1 >	REAL*4 >  intersect with Earth, if there are
!			Z1/	REAL*4/	  any; the nearer point of two
!			X2\	REAL*4\	GSEq position of second line-of-sight
!			Y2 >	REAL*4 >  intersect with Earth is there is
!			Z2/	REAL*4/	  one; the farther point
!
!
        COMMON /IMCOORD/IMXMIN,IMXMAX,IMYMAX,STEP,PIXWD,PIXHT,
     &                  SP,ALTF,NSPIN,TSPIN,UTNC1,SOLMSEC1,MDIREC
	COMMON /MOTION/	POSITION,SPINANG,XP1,SPINX,SPINY,SPINZ,
     &			SUNX,SUNZ
!
	INTEGER*2	NP,IPOS,MDIREC,XPOS
	INTEGER*4	UTNC1,NSPIN,TSPIN,SOLMSEC1
	REAL*4		IMXMIN,IMXMAX,IMYMAX,STEP,PIXWD,PIXHT,
     &			POSITION(1026,3),SPINANG(1026),XP1,
     &			SPINX,SPINY,SPINZ,SUNX,SUNZ,XS,YS,ALT,
     &			A,B,SCX,SCY,SCZ,GX,GY,GZ,QA,QB,QC,SP,ALTF,
     &			X1,Y1,Z1,X2,Y2,Z2,QX,QY,QZ,Q1,Q2
	REAL*8		AXA,BXB,DX,DY,DZ,U,V
!
!---screen out sight lines away from Earth
	IF (ABS(YS*PIXHT).GT.90.) THEN
	    NP = 0
	    RETURN
	ENDIF
!
!---set up earth size
	A = 6378.164 + ALT
	B = 6356.779 + ALT
	AXA = DBLE(A)*DBLE(A)
	BXB = DBLE(B)*DBLE(B)
!
!---get look direction in GSEq coordinates
	CALL VW2GSESC(XS,YS,GX,GY,GZ)
	IPOS = XPOS(XS,YS)
!
!---use quadratic solution to get intersections of line of sight
!		with ellipsoid
	DX = DBLE(GX)
	DY = DBLE(GY)
	DZ = DBLE(GZ)
	IF (ABS(GY).GT.ABS(GX).OR.ABS(GZ).GT.ABS(GX)) GO TO 20
!---solve for X
	U = POSITION(IPOS,2) - (DY/DX)*POSITION(IPOS,1)
	V = POSITION(IPOS,3) - (DZ/DX)*POSITION(IPOS,1)
	QA = (1.D0 + (DY*DY)/(DX*DX))/AXA + (DZ*DZ)/(DX*DX)/BXB
	QB = 2.D0 * (DY/DX*U/AXA + DZ/DX*V/BXB)
	QC = (U*U)/AXA + (V*V)/BXB - 1.D0
	CALL QUAD(QA,QB,QC,X1,X2,NP)
	IF (NP.EQ.0) RETURN
	Y1 = (DY/DX)*(X1-POSITION(IPOS,1)) + POSITION(IPOS,2)
	Z1 = (DZ/DX)*(X1-POSITION(IPOS,1)) + POSITION(IPOS,3)
	IF (NP.EQ.1) RETURN
	Y2 = (DY/DX)*(X2-POSITION(IPOS,1)) + POSITION(IPOS,2)
	Z2 = (DZ/DX)*(X2-POSITION(IPOS,1)) + POSITION(IPOS,3)
	GO TO 70
 20	IF (ABS(GZ).GT.ABS(GY)) GO TO 30
!---solve for Y
	U = POSITION(IPOS,1) - (DX/DY)*POSITION(IPOS,2)
	V = POSITION(IPOS,3) - (DZ/DY)*POSITION(IPOS,2)
	QA = (1.D0 + (DX*DX)/(DY*DY))/AXA + (DZ*DZ)/(DY*DY)/BXB
	QB = 2.D0 * (DX/DY*U/AXA + DZ/DY*V/BXB)
	QC = (U*U)/AXA + (V*V)/BXB - 1.D0
	CALL QUAD(QA,QB,QC,Y1,Y2,NP)
	IF (NP.EQ.0) RETURN
	X1 = (DX/DY)*(Y1-POSITION(IPOS,2)) + POSITION(IPOS,1)
	Z1 = (DZ/DY)*(Y1-POSITION(IPOS,2)) + POSITION(IPOS,3)
	IF (NP.EQ.1) RETURN
	X2 = (DX/DY)*(Y2-POSITION(IPOS,2)) + POSITION(IPOS,1)
	Z2 = (DZ/DY)*(Y2-POSITION(IPOS,2)) + POSITION(IPOS,3)
	GO TO 70
!---solve for Z
 30	U = POSITION(IPOS,1) - (DX/DZ)*POSITION(IPOS,3)
	V = POSITION(IPOS,2) - (DY/DZ)*POSITION(IPOS,3)
	QA = ((DX*DX)/(DZ*DZ)+(DY*DY)/(DZ*DZ))/AXA + 1.D0/BXB
	QB = 2.D0/AXA * (DX/DZ*U + DY/DZ*V)
	QC = (U*U + V*V)/AXA - 1.D0
	CALL QUAD(QA,QB,QC,Z1,Z2,NP)
	IF (NP.EQ.0) RETURN
	Y1 = (DY/DZ)*(Z1-POSITION(IPOS,3)) + POSITION(IPOS,2)
	X1 = (DX/DZ)*(Z1-POSITION(IPOS,3)) + POSITION(IPOS,1)
	IF (NP.EQ.1) RETURN
	Y2 = (DY/DZ)*(Z2-POSITION(IPOS,3)) + POSITION(IPOS,2)
	X2 = (DX/DZ)*(Z2-POSITION(IPOS,3)) + POSITION(IPOS,1)
!
!---select the solution point closer to the s/c for point 1
 70	QX = X1 - POSITION(IPOS,1)
	QY = Y1 - POSITION(IPOS,2)
	QZ = Z1 - POSITION(IPOS,3)
	Q1 = SQRT( QX*QX + QY*QY + QZ*QZ )
	QX = X2 - POSITION(IPOS,1)
	QY = Y2 - POSITION(IPOS,2)
	QZ = Z2 - POSITION(IPOS,3)
	Q2 = SQRT( QX*QX + QY*QY + QZ*QZ )
	IF (Q1.LT.Q2) RETURN
	QX = X1
	QY = Y1
	QZ = Z1
	X1 = X2
	Y1 = Y2
	Z1 = Z2
	X2 = QX
	Y2 = QY
	Z2 = QZ
	RETURN
!
	END
!
!
!
	SUBROUTINE VW2GEO(XS,YS,ALT,GLAT,GLONG,GLT,GOOD)
!
!  converts view coordinates XS and YS to geographic N LAT
!  and E LONG in degrees assuming altitude of ALT kilometers above
!  the Earth; also provides geographic local time; sets GOOD to
!  .FALSE. if line of sight does not intersect the Earth
!
!			name	type	description
!
!   input arguments:	XS	REAL*4	horizontal view coordinate
!			YS	REAL*4	vertical view coordinate
!			ALT	REAL*4	altitude in kilometers
!  output arguments:	GLAT	REAL*4	geographic north latitude in deg. 
!			GLONG	REAL*4	geographic east longitude in deg.
!			GLT	REAL*4	geographic local time in degrees
!			GOOD	LOG*1	.TRUE. if line of sight intersects
!					  Earth; .FALSE. if it does not
!
!
        COMMON /IMCOORD/IMXMIN,IMXMAX,IMYMAX,STEP,PIXWD,PIXHT,
     &                  SP,ALTF,NSPIN,TSPIN,UTNC1,SOLMSEC1,MDIREC
!
	INTEGER*2	MDIREC,NP
	INTEGER*4	UTNC1,NSPIN,TSPIN,SOLMSEC,SOLMSEC1
	REAL*4		IMXMIN,IMXMAX,IMYMAX,STEP,PIXWD,PIXHT,SP,ALTF,
     &			XS,YS,ALT,GX1,GY1,GZ1,GX2,GY2,GZ2,GR,GTHETA,
     &			GPHI,GLT,GLAT,GLONG,X1
	LOGICAL*1	GOOD
!
!---get GSEq coordinates of the view point
	CALL POINT(XS,YS,ALT,NP,GX1,GY1,GZ1,GX2,GY2,GZ2)
	IF (NP.EQ.0) THEN
	    GOOD = .FALSE.
	    RETURN
	ELSE
	    GOOD = .TRUE.
	ENDIF
!
!---compute local geographic time
	CALL RECSPHD(GX1,GY1,GZ1,GR,GTHETA,GPHI)
	GLT = AMOD(GPHI+180.,360.)
!
!---convert GSEq coordinates to latitude and longitude
	IF (MDIREC.EQ.1) THEN
	    X1 = IMXMIN
	ELSE
	    X1 = IMXMAX
	ENDIF
	SOLMSEC = SOLMSEC1 + (XS-X1)*MDIREC*NSPIN 
	CALL GSEGEO(GX1,GY1,GZ1,SOLMSEC,GLAT,GLONG)
	RETURN
!
	END
!
!
!
	SUBROUTINE VW2GEI(XS,YS,ALT,XGEI,YGEI,ZGEI,GOOD)
!
!  converts view coordinates XS and YS to geocentric equatorial
!  inertial (GEI) coordinates at the altitude of ALT kilometers above the
!  Earth; sets GOOD to .FALSE. if line of sight does not intersect the Earth
!
!			name	type	description
!
!   input arguments:	XS	REAL*4	horizontal view coordinate
!			YS	REAL*4	vertical view coordinate
!			ALT	REAL*4	altitude in kilometers
!  output arguments:	XGEI	REAL*4\
!			YGEI	REAL*4 >GEI position vector in km.
!			ZGEI	REAL*4/
!			GOOD	LOG*1	.TRUE. if line of sight intersects
!					  Earth; .FALSE. if it does not
!
!
	COMMON /MAFREC/	HEADER,SCAN
!
	BYTE		HEADER(404),SCAN(174)
	INTEGER*4	SUN(3)
	INTEGER*2	NP
	REAL*4		XS,YS,ALT,GX1,GY1,GZ1,GX2,GY2,GZ2,
     &			XGEI,YGEI,ZGEI
	LOGICAL*1	GOOD
!
	EQUIVALENCE (HEADER(173),SUN(1))
!
!---get GSEq coordinates of the view point
	CALL POINT(XS,YS,ALT,NP,GX1,GY1,GZ1,GX2,GY2,GZ2)
	IF (NP.EQ.0) THEN
	    GOOD = .FALSE.
	    RETURN
	ELSE
	    GOOD = .TRUE.
	ENDIF
!
!---convert GSEq coordinates to GEI coordinates
	CALL GSEGEI(GX1,GY1,GZ1,ATAN2D(SUN(2)/1.E6,SUN(1)/1.E6),
     &		XGEI,YGEI,ZGEI)
!
	RETURN
	END
!
!
!
	SUBROUTINE GEO2VW(GLAT,GLONG,ALT,XS,YS,VIS)
!
!  converts geographic N LAT and E LONG in degrees to view coordinates;
!  VIS is set to .FALSE. in GSE2VW if the point is not visible
!
!			name	type	description
!
!   input arguments:	GLAT	REAL*4	geographic north latitude in deg. 
!			GLONG	REAL*4	geographic east longitude in deg.
!			ALT	REAL*4	altitude in kilometers
!  output arguments:	XS	REAL*4	horizontal view coordinate
!			YS	REAL*4	vertical view coordinate
!			VIS	LOG*1	.TRUE. if input point is visible
!					  to spacecraft; .FALSE. if not
!
!
        COMMON /IMCOORD/IMXMIN,IMXMAX,IMYMAX,STEP,PIXWD,PIXHT,
     &                  SP,ALTF,NSPIN,TSPIN,UTNC1,SOLMSEC1,MDIREC
	COMMON /MOTION/	POSITION,SPINANG,XP1,SPINX,SPINY,SPINZ,
     &			SUNX,SUNZ
!
	INTEGER*2	MDIREC
	INTEGER*4	UTNC1,NSPIN,TSPIN,SOLMSEC1
	REAL*4		IMXMIN,IMXMAX,IMYMAX,STEP,PIXWD,PIXHT,
     &			POSITION(1026,3),SPINANG(1026),XP1,SP,ALTF,
     &			SPINX,SPINY,SPINZ,SUNX,SUNZ,GLAT,GLONG,ALT,
     &			XS,YS,RLONG,XSL,XSLL,X,Y,Z,RAD,RADIUS
	LOGICAL*1	VIS
!
!---convert lat and long to GSEq coordinates
	RLONG = GLONG + 360.*SOLMSEC1/86400000. + 180.
	RLONG = AMOD(RLONG,360.)
	IF (MDIREC.EQ.1) THEN
	    XSL = IMXMIN
	ELSE
	    XSL = IMXMAX
	ENDIF
	XSLL = XSL
!
 12	CALL SPHDREC(1.,90.-GLAT,RLONG,X,Y,Z)
!
!---put point at input altitude
	RAD = RADIUS(GLAT)
	X = (RAD+ALT)*X
	Y = (RAD+ALT)*Y
	Z = (RAD+ALT)*Z
!
!---convert from GSEq to view coordinates
	CALL GSE2VW(X,Y,Z,XS,YS,VIS)
!
!---correct for Earth's rotation during image time
	IF (ABS(XS-XSL).LT.0.1) RETURN
	IF (ABS(XS-XSLL).LT.0.1) RETURN
	RLONG = RLONG + .41666667E-5*(XS-XSL)*NSPIN*MDIREC
	XSLL = XSL
	XSL = XS
	GO TO 12
!
	END
!
!
!
	INTEGER*2 FUNCTION XPOS(XS,YS)
!
!  for the window X-coordinate XS, returns the index of the closest
!  s/c position
!
!			name	type	description
!
!   input arguments:	XS	REAL*4	horizontal view coordinate
!			YS	REAL*4	vertical view coordinate
!  	     result:	XPOS	INT*2	index of closest position in
!					  POSITION array
!
!
	COMMON /MOTION/	POSITION,SPINANG,XP1,SPINX,SPINY,SPINZ,
     &			SUNX,SUNZ
!
	REAL*4		POSITION(1026,3),SPINANG(1026),XP1,
     &			SPINX,SPINY,SPINZ,SUNX,SUNZ,XS,YS,X,TRUEX
!
	X = TRUEX(XS,YS)
	XPOS = NINT(X-XP1) + 1
!
	RETURN
	END
!
!
!
	SUBROUTINE GSESCPOS(XGSEQ,YGSEQ,ZGSEQ,IPOS)
!
!  for the line of sight to the input GSEq point, returns the angle
!  from the spin vector and the index of the closest s/c position in
!  the POSITION array
!
!			name	type	description
!
!   input arguments:	XGSEQ	REAL*4\ 
!			YGSEQ	REAL*4 >GSEq position vector in km.
!			ZGSEQ	REAL*4/
!  output arguments:	IPOS	INT*2	index of closest position in
!					  POSITION array
!
	COMMON /WINDOW/	WNXMIN,WNXMAX,WNYMIN,WNYMAX
	COMMON /MOTION/	POSITION,SPINANG,XP1,SPINX,SPINY,SPINZ,
     &			SUNX,SUNZ
!
	INTEGER*2	LASTI,I,IPOS,OUT
	REAL*4		WNXMIN,WNXMAX,WNYMIN,WNYMAX,
     &			POSITION(1026,3),SPINANG(1026),XP1,
     &			SPINX,SPINY,SPINZ,SUNX,SUNZ,XGSEQ,YGSEQ,ZGSEQ
!
	LASTI = NINT((WNXMAX-WNXMIN)/2.)
	I = LASTI
	OUT = 0
 30	CALL SCPOS(XGSEQ-POSITION(I,1),YGSEQ-POSITION(I,2),
     &			ZGSEQ-POSITION(I,3),IPOS)
	IF (IPOS.EQ.I) RETURN
	IF (IPOS.EQ.LASTI) RETURN
	LASTI = I
	I = IPOS
	IF (IPOS.LT.1) THEN
	    OUT = OUT + 1
	    IF (OUT.EQ.2) RETURN
	    I = 1
	ENDIF
	IF (IPOS.GT.NINT(WNXMAX-XP1)+2) THEN
	    OUT = OUT + 1
	    IF (OUT.EQ.2) RETURN
	    I = NINT(WNXMAX-XP1)+2
	ENDIF
	GO TO 30
!
	END
!
!
!
	SUBROUTINE SCPOS(GX,GY,GZ,IPOS)
!
!  computes the index of the closest s/c position in the POSITION array
!  for the input GSEq vector
!  
!			name	type	description
!
!   input arguments:	GX	REAL*4\ 
!			GY	REAL*4 >GSEq direction vector
!			GZ	REAL*4/
!   output argument:	IPOS	INT*2	index of closest position in
!					  POSITION array
!
!
        COMMON /IMAGE/  IMFILE,FCTDWN,FCTACR,NPHOT,CENTRMLC,PIXOFF
        COMMON /IMCOORD/IMXMIN,IMXMAX,IMYMAX,STEP,PIXWD,PIXHT,
     &                  SP,ALTF,NSPIN,TSPIN,UTNC1,SOLMSEC1,MDIREC
	COMMON /WINDOW/	WNXMIN,WNXMAX,WNYMIN,WNYMAX
	COMMON /MOTION/	POSITION,SPINANG,XP1,SPINX,SPINY,SPINZ,
     &			SUNX,SUNZ
	COMMON /MAFREC/	HEADER,SCAN
!
	BYTE		HEADER(404),SCAN(174)
	CHARACTER*128	IMFILE
	INTEGER*2	NPHOT,CENTRMLC,PIXOFF,MDIREC,IPOS,LCENTR(3)
	INTEGER*4	UTNC1,NSPIN,TSPIN,FIRSTMLC,LASTMLC,SOLMSEC1
	REAL*4		FCTDWN,FCTACR,IMXMIN,IMXMAX,IMYMAX,STEP,
     &			PIXWD,PIXHT,WNXMIN,WNXMAX,WNYMIN,WNYMAX,
     &			POSITION(1026,3),SPINANG(1026),XP1,SP,ALTF,
     &			SPINX,SPINY,SPINZ,SUNX,SUNZ,GX,GY,GZ,
     &			GXN,GYN,GZN,XC,X
	REAL*8		ZANG
!
	EQUIVALENCE (HEADER(41),FIRSTMLC),(HEADER(45),LASTMLC)
	DATA LCENTR/80,73,90/
!
	CALL NRMLIZ(GX,GY,GZ,GXN,GYN,GZN)
	ZANG = DACOSD( DBLE(GXN)*SPINX+DBLE(GYN)*SPINY+DBLE(GZN)*SPINZ )
	IF (NPHOT.EQ.3) THEN
	    XC = IMXMIN + (LCENTR(3)-MIN0(FIRSTMLC,LASTMLC))
	ELSE
	    XC = IMXMIN + (MAX0(FIRSTMLC,LASTMLC)-LCENTR(NPHOT))
	ENDIF
	X = XC - (90.-ZANG)/PIXWD
	IPOS = NINT(X-XP1) + 1
	RETURN
	END
!
!
!
	SUBROUTINE LMPLAN(IPOS,PLX,PLY,PLZ)
!
!  for the s/c position indicated by IPOS, returns intercept coordinates
!  at polar plane of line through the s/c position and normal to the
!  polar plane (polar plane includes the limb)
!
!			name	type	description
!
!   input arguments:	IPOS	INT*2	index in POSITION array
!  output arguments:	PLX	REAL*4\ GSEq coordinates of intersection
!			PLY	REAL*4 >  of normal from s/c position and
!			PLZ	REAL*4/   polar plane
!
!
	COMMON /MOTION/	POSITION,SPINANG,XP1,SPINX,SPINY,SPINZ,
     &			SUNX,SUNZ
!
	INTEGER*2	IPOS
	REAL*4		POSITION(1026,3),SPINANG(1026),XP1,
     &			SPINX,SPINY,SPINZ,SUNX,SUNZ,RDMAX,RDMIN,
     &			PLXS(1026),PLYS(1026),PLZS(1026),PLX,PLY,PLZ
	REAL*8		A,B,C,D,AN,BN,CN
!
	DATA RDMAX/6378.164/,RDMIN/6356.779/
!
	IF (PLXS(IPOS).EQ.0..AND.PLYS(IPOS).EQ.0..AND.PLZS(IPOS).EQ.0.)
     &					GO TO 23
 12	PLX = PLXS(IPOS)
	PLY = PLYS(IPOS)
	PLZ = PLZS(IPOS)
	RETURN
!
!---(A,B,C) is normal vector to polar plane
 23	A = DBLE(POSITION(IPOS,1))/(DBLE(RDMAX)*DBLE(RDMAX))
	B = DBLE(POSITION(IPOS,2))/(DBLE(RDMAX)*DBLE(RDMAX))
	C = DBLE(POSITION(IPOS,3))/(DBLE(RDMIN)*DBLE(RDMIN))
	CALL DNRMLIZ(A,B,C,AN,BN,CN)
!
!---intercept point along normal line between s/c and polar plane
!		is (SCPOSX,SCPOSY,SCPOSZ) - (A,B,C)*d  where d is the
!		distance from the s/c to the plane
	D = (A*POSITION(IPOS,1)+B*POSITION(IPOS,2)+C*POSITION(IPOS,3)-1.)
     &			/DSQRT(A*A+B*B+C*C)
	PLXS(IPOS) = SNGL( POSITION(IPOS,1) - AN*D )
	PLYS(IPOS) = SNGL( POSITION(IPOS,2) - BN*D )
	PLZS(IPOS) = SNGL( POSITION(IPOS,3) - CN*D )
	GO TO 12
!
	END
!
!
!
	SUBROUTINE GSESC2VW(GX,GY,GZ,IPOS,XS,YS)
!
!  rotates input GSEq vector from the s/c to view coordinates based on position
!  indicated by IPOS
!
!			name	type	description
!
!   input arguments:	GX	REAL*4\ 
!			GY	REAL*4 >GSEq vector
!			GZ	REAL*4/
!			IPOS	INT*2	index of s/c position in
!					  POSITION array
!  output arguments:	XS	REAL*4	horizontal view coordinate
!			YS	REAL*4	vertical view coordinate
!
!
!
	COMMON /MOTION/	POSITION,SPINANG,XP1,SPINX,SPINY,SPINZ,
     &			SUNX,SUNZ
        COMMON /IMCOORD/IMXMIN,IMXMAX,IMYMAX,STEP,PIXWD,PIXHT,
     &                  SP,ALTF,NSPIN,TSPIN,UTNC1,SOLMSEC1,MDIREC
!
	INTEGER*2	IPOS,MDIREC
	REAL*4		POSITION(1026,3),SPINANG(1026),XP1,
     &			SPINX,SPINY,SPINZ,SUNX,SUNZ,GX,GY,GZ,XX,XY,XZ,
     &			YYX,YYY,YYZ,YX,YY,YZ,ZZX,ZZY,ZZZ,ZX,ZY,ZZ,
     &			VX,VY,VZ,VR,VTHETA,VPHI,TRUEXS,XS,YS,IMXMIN,
     &			IMXMAX,IMYMAX,STEP,PIXWD,PIXHT,SP,ALTF
	INTEGER*4	NSPIN,TSPIN,UTNC1,SOLMSEC1
!
!---set up rotation matrix -
!	 X-axis is toward earth center, Y-axis is X-axis cross spin vector
	CALL NRMLIZ(-POSITION(IPOS,1),-POSITION(IPOS,2),
     &			-POSITION(IPOS,3),XX,XY,XZ)
	CALL CROSS(XX,XY,XZ,SPINX,SPINY,SPINZ,YYX,YYY,YYZ)
	CALL NRMLIZ(YYX,YYY,YYZ,YX,YY,YZ)
	CALL CROSS(XX,XY,XZ,YX,YY,YZ,ZZX,ZZY,ZZZ)
	CALL NRMLIZ(ZZX,ZZY,ZZZ,ZX,ZY,ZZ)
!
!---rotate vector
	VX = GX*XX + GY*XY + GZ*XZ
	VY = GX*YX + GY*YY + GZ*YZ
	VZ = GX*ZX + GY*ZY + GZ*ZZ
!
!---convert to screen angles
	CALL RECSPHD(VX,VY,VZ,VR,VTHETA,VPHI)
	IF (VPHI.GT.180.) VPHI = VPHI - 360.
	YS = VPHI/PIXHT
	XS = TRUEXS((90.-VTHETA)/PIXWD,YS)
!
	RETURN
	END
!
!
!
	SUBROUTINE VW2GSESC(XS,YS,GX,GY,GZ)
!
!  finds GSEq vector for the look direction from the s/c indicated by
!  the input screen coordinates
!
!			name	type	description
!
!   input arguments:	XS	REAL*4	horizontal view coordinate
!			YS	REAL*4	vertical view coordinate
!  output arguments:	GX	REAL*4\ 
!			GY	REAL*4 >GSEq vector
!			GZ	REAL*4/
!
!
	COMMON /MOTION/	POSITION,SPINANG,XP1,SPINX,SPINY,SPINZ,
     &			SUNX,SUNZ
        COMMON /IMCOORD/IMXMIN,IMXMAX,IMYMAX,STEP,PIXWD,PIXHT,
     &                  SP,ALTF,NSPIN,TSPIN,UTNC1,SOLMSEC1,MDIREC
!
	INTEGER*2	IPOS,XPOS,MDIREC
	REAL*4		POSITION(1026,3),SPINANG(1026),XP1,
     &			SPINX,SPINY,SPINZ,SUNX,SUNZ,GX,GY,GZ,XX,XY,XZ,
     &			YYX,YYY,YYZ,YX,YY,YZ,ZZX,ZZY,ZZZ,ZX,ZY,ZZ,
     &			VX,VY,VZ,VTHETA,VPHI,XS,YS,TRUEX,IMXMIN,IMXMAX,
     &			IMYMAX,STEP,PIXWD,PIXHT,SP,ALTF
	INTEGER*4	NSPIN,TSPIN,UTNC1,SOLMSEC1
!
!---get position index
	IPOS = XPOS(XS,YS)
!
!---set up rotation matrix -
!	 X-axis is toward earth center, Y-axis is X-axis cross spin vector
	CALL NRMLIZ(-POSITION(IPOS,1),-POSITION(IPOS,2),
     &			-POSITION(IPOS,3),XX,XY,XZ)
	CALL CROSS(XX,XY,XZ,SPINX,SPINY,SPINZ,YYX,YYY,YYZ)
	CALL NRMLIZ(YYX,YYY,YYZ,YX,YY,YZ)
	CALL CROSS(XX,XY,XZ,YX,YY,YZ,ZZX,ZZY,ZZZ)
	CALL NRMLIZ(ZZX,ZZY,ZZZ,ZX,ZY,ZZ)
!
!---convert screen angles to view vector
	VPHI = YS*PIXHT
	VTHETA = 90. - TRUEX(XS,YS)*PIXWD
	CALL SPHDREC(1.,VTHETA,VPHI,VX,VY,VZ)
!
!---rotate vector
	GX = VX*XX + VY*YX + VZ*ZX
	GY = VX*XY + VY*YY + VZ*ZY
	GZ = VX*XZ + VY*YZ + VZ*ZZ
!
	RETURN
	END
!
!
!
	REAL*4 FUNCTION TRUEX(XS,YS)
!
!  corrects view X coordinate for the mirror step to get true angle
!
!			name	type	description
!
!   input arguments:	XS	REAL*4	horizontal view coordinate
!			YS	REAL*4	vertical view coordinate
!	     result:	TRUEX	REAL*4	horizontal view coordinate
!					  adjusted for mirror step to
!					  correct from scanline position
!					  to true angle
!
!
        COMMON /IMCOORD/IMXMIN,IMXMAX,IMYMAX,STEP,PIXWD,PIXHT,
     &                  SP,ALTF,NSPIN,TSPIN,UTNC1,SOLMSEC1,MDIREC
!
	REAL*4		XS,YS,IMXMIN,IMXMAX,IMYMAX,STEP,PIXWD,PIXHT,
     &			SP,ALTF
	INTEGER*2	MDIREC
	INTEGER*4	NSPIN,TSPIN,SOLMSEC1,UTNC1
!
	TRUEX = XS
	IF (-YS*PIXHT.GT.STEP) TRUEX = TRUEX + MDIREC
!
	RETURN
	END
!
!
!
	REAL*4 FUNCTION TRUEXS(X,YS)
!
!  corrects X coordinate for the mirror step to get actual view
!  position
!
!			name	type	description
!
!   input arguments:	X	REAL*4	horizontal view component
!					  based on true sight angle
!			YS	REAL*4	vertical view coordinate
!	     result:	TRUEXS	REAL*4	horizontal view coordinate
!					  after adjustment for mirror step
!					  to actual scanline position
!
!
        COMMON /IMCOORD/IMXMIN,IMXMAX,IMYMAX,STEP,PIXWD,PIXHT,
     &                  SP,ALTF,NSPIN,TSPIN,UTNC1,SOLMSEC1,MDIREC
!
	REAL*4		X,YS,IMXMIN,IMXMAX,IMYMAX,STEP,PIXWD,PIXHT,
     &			SP,ALTF
	INTEGER*2	MDIREC
	INTEGER*4	NSPIN,TSPIN,UTNC1,SOLMSEC1
!
	TRUEXS = X
	IF (-YS*PIXHT.GT.STEP) TRUEXS = TRUEXS - MDIREC
!
	RETURN
	END
!
!
!
	INTEGER*4 FUNCTION UTPIX(XS,YS)
!
!  returns the UT in milliseconds for the collection time of the
!  pixel at the input view position
!
!			name	type	description
!
!   input arguments:	XS	REAL*4	horizontal view coordinate
!			YS	REAL*4	vertical view coordinate
!	     result:	UTPIX	INT*4	pixel collection UT in msec.
!
!
        COMMON /IMCOORD/IMXMIN,IMXMAX,IMYMAX,STEP,PIXWD,PIXHT,
     &                  SP,ALTF,NSPIN,TSPIN,UTNC1,SOLMSEC1,MDIREC
	COMMON /MOTION/	POSITION,SPINANG,XP1,SPINX,SPINY,SPINZ,
     &			SUNX,SUNZ
!
	REAL*4		XS,YS,IMXMIN,IMXMAX,IMYMAX,STEP,PIXWD,PIXHT,
     &			POSITION(1026,3),SPINANG(1026),XP1,SP,ALTF,
     &			SPINX,SPINY,SPINZ,SUNX,SUNZ
	INTEGER*2	MDIREC
	INTEGER*4	NSPIN,TSPIN,UTNC1,SOLMSEC1
!
	IF (MDIREC.EQ.1) THEN
	    UTPIX = UTNC1 + (XS-IMXMIN)*NSPIN - YS*3.90625
	ELSE
	    UTPIX = UTNC1 + (IMXMAX-XS)*NSPIN - YS*3.90625
	ENDIF
!
	RETURN
	END
